@@ -1,6 +1,7 @@
 from contextlib import contextmanager
 import dataclasses
 import functools
+from hashlib import sha256
 import importlib.abc
 import importlib.util
 import itertools
@@ -9,15 +10,38 @@ import os
 import shutil
 import subprocess
 import sys
+import tempfile
 import traceback
 import typing as t
 
 import click
+import pexpect
+from rich import print as rich_print
+from rich.pretty import Pretty
+from rich.status import Status
+from rich.table import Table
 
 logger = logging.getLogger(__name__)
 
 SHELL = os.getenv("SHELL", "/bin/bash")
 ENCODING = sys.getdefaultencoding()
+SHELL_RCFILE = """
+source {venv_path}/bin/activate
+echo -e "\e[31;1m"
+echo "                 )  "
+echo " (   (        ( /(  "
+echo " )(  )\   (   )\()) "
+echo "(()\((_)  )\ (_))/  "
+echo " ((_)(_) ((_)| |_   "
+echo "| '_|| |/ _ \|  _|  "
+echo "|_|  |_|\___/ \__|  "
+echo -e "\e[0m"
+echo -e "\e[33;1mInteractive shell\e[0m"
+echo ""
+echo -e "* Venv name   : \e[1m{name}\e[0m"
+echo -e "* Venv path   : \e[1m{venv_path}\e[0m"
+echo -e "* Interpreter : \e[1m$( python -V )\e[0m"
+"""
 
 
 if t.TYPE_CHECKING or sys.version_info[:2] >= (3, 9):
@@ -311,6 +335,15 @@ class VenvInstance:
                     break
                 ancestor = ancestor.parent
 
+    def matches_pattern(self, pattern: t.Pattern[str]) -> bool:
+        """Return whether this VenvInstance matches the provided pattern.
+
+        The pattern is checked against the instance ``name`` and ``short_hash``.
+        """
+        if self.name and pattern.match(self.name):
+            return True
+        return bool(pattern.match(self.short_hash))
+
     @property
     def prefix(self) -> t.Optional[str]:
         """Return path to directory where dependencies should be installed.
@@ -349,7 +382,10 @@ class VenvInstance:
         if not self.pkgs:
             return None
         return "_".join(
-            (f"{n}{rmchars('<=>.,', v)}" for n, v in sorted(self.pkgs.items()))
+            (
+                f"{rmchars('<=>.,:+@/', n)}{rmchars('<=>.,:+@/', v)}"
+                for n, v in self.pkgs.items()
+            )
         )
 
     @property
@@ -372,6 +408,17 @@ class VenvInstance:
 
         return pip_deps(pkgs)
 
+    @property
+    def short_hash(self) -> str:
+        return hex(hash(self))[2:9]
+
+    def __hash__(self):
+        """Compute a hash for the venv instance."""
+        h = sha256()
+        h.update(repr(self).encode())
+        h.update(self.full_pkg_str.encode())
+        return int(h.hexdigest(), 16)
+
     @property
     def bin_path(self) -> t.Optional[str]:
         prefix = self.prefix
@@ -586,7 +633,7 @@ class Session:
                 logger.debug("Skipping venv instance %s due to missing command", inst)
                 continue
 
-            if inst.name and not pattern.match(inst.name):
+            if inst.name and not inst.matches_pattern(pattern):
                 logger.debug(
                     "Skipping venv instance %s due to name pattern mismatch.", inst
                 )
@@ -627,6 +674,20 @@ class Session:
             else:
                 env = dict(inst.env)
 
+            # Add riot specific environment variables
+            env.update(
+                {
+                    "RIOT": "1",
+                    "RIOT_PYTHON_HINT": str(inst.py),
+                    "RIOT_PYTHON_VERSION": inst.py.version(),
+                    "RIOT_VENV_HASH": inst.short_hash,
+                    "RIOT_VENV_IDENT": inst.ident or "",
+                    "RIOT_VENV_NAME": inst.name or "",
+                    "RIOT_VENV_PKGS": inst.pkg_str,
+                    "RIOT_VENV_FULL_PKGS": inst.full_pkg_str,
+                }
+            )
+
             inst.prepare(env, recreate=recreate_venvs, skip_deps=skip_base_install)
 
             pythonpath = inst.pythonpath
@@ -694,7 +755,7 @@ class Session:
         for r in results:
             failed = r.code != 0
             env_str = env_to_str(r.instance.env)
-            s = f"{r.instance.name}: {env_str} python{r.instance.py} {r.instance.full_pkg_str}"
+            s = f"{r.instance.name}: [{r.instance.short_hash}] {env_str} python{r.instance.py} {r.instance.full_pkg_str}"
 
             if failed:
                 num_failed += 1
@@ -716,9 +777,23 @@ class Session:
         if any(True for r in results if r.code != 0):
             sys.exit(1)
 
-    def list_venvs(self, pattern, venv_pattern, pythons=None, out=sys.stdout):
-        for inst in self.venv.instances():
-            if not inst.name or not pattern.match(inst.name):
+    def list_venvs(
+        self, pattern, venv_pattern, pythons=None, out=sys.stdout, pipe_mode=False
+    ):
+        table = None
+        if not pipe_mode:
+            table = Table(
+                "No.",
+                "Hash",
+                "Name",
+                "Interpreter",
+                "Environment",
+                "Packages",
+                box=None,
+            )
+
+        for n, inst in enumerate(self.venv.instances()):
+            if not inst.name or not inst.matches_pattern(pattern):
                 continue
 
             if pythons and inst.py not in pythons:
@@ -728,8 +803,22 @@ class Session:
                 continue
             pkgs_str = inst.full_pkg_str
             env_str = env_to_str(inst.env)
-            py_str = f"Python {inst.py}"
-            click.echo(f"{inst.name} {env_str} {py_str} {pkgs_str}")
+            if pipe_mode:
+                print(
+                    f"[#{n}]  {inst.short_hash}  {inst.name:12} {env_str} {inst.py} Packages({pkgs_str})"
+                )
+            else:
+                table.add_row(
+                    f"[cyan]#{n}[/cyan]",
+                    f"[bold cyan]{hex(hash(inst))[2:9]}[/bold cyan]",
+                    f"[bold]{inst.name}[/bold]",
+                    Pretty(inst.py),
+                    env_str or "--",
+                    f"[italic]{pkgs_str}[/italic]",
+                )
+
+        if table:
+            rich_print(table)
 
     def generate_base_venvs(
         self,
@@ -744,7 +833,8 @@ class Session:
             [
                 inst.py
                 for inst in self.venv.instances()
-                if inst.py is not None and (not inst.name or pattern.match(inst.name))
+                if inst.py is not None
+                and (not inst.name or inst.matches_pattern(pattern))
             ]
         )
         # Apply Python filters.
@@ -771,6 +861,79 @@ class Session:
                 # Install the dev package into the base venv.
                 install_dev_pkg(venv_path)
 
+    def _generate_shell_rcfile(self):
+        with tempfile.NamedTemporaryFile() as rcfile:
+            rcfile.write()
+            rcfile.flush()
+
+    def shell(self, ident, pass_env):
+        for n, inst in enumerate(self.venv.instances()):
+            if ident != f"#{n}" and not hex(hash(inst))[2:].startswith(ident):
+                continue
+
+            assert inst.py is not None, inst
+            try:
+                venv_path = inst.venv_path
+            except FileNotFoundError:
+                raise RuntimeError("%s not available" % inst.py)
+
+            logger.info("Launching shell inside venv instance %s", inst)
+            logger.debug("Setting venv path to %s", venv_path)
+
+            # Generate the environment for the instance.
+            if pass_env:
+                env = os.environ.copy()
+                env.update(dict(inst.env))
+            else:
+                env = dict(inst.env)
+
+            # Should we expect the venv to be ready?
+            with Status("Preparing shell virtual environment"):
+                inst.py.create_venv(False)
+                inst.prepare(env)
+
+            pythonpath = inst.pythonpath
+            if pythonpath:
+                env["PYTHONPATH"] = (
+                    f"{pythonpath}:{env['PYTHONPATH']}"
+                    if "PYTHONPATH" in env
+                    else pythonpath
+                )
+            script_path = inst.scriptpath
+            if script_path:
+                env["PATH"] = ":".join(
+                    (script_path, env.get("PATH", os.environ["PATH"]))
+                )
+
+            with nspkgs(inst):
+                with tempfile.NamedTemporaryFile() as rcfile:
+                    rcfile.write(
+                        SHELL_RCFILE.format(
+                            venv_path=venv_path, name=inst.name
+                        ).encode()
+                    )
+                    rcfile.flush()
+
+                    try:
+                        w, h = os.get_terminal_size()
+                    except OSError:
+                        w, h = 80, 24
+                    c = pexpect.spawn(SHELL, ["-i"], dimensions=(h, w), env=env)
+                    c.setecho(False)
+                    c.sendline(f"source {rcfile.name}")
+                    try:
+                        c.interact()
+                    except Exception:
+                        pass
+                    c.close()
+                    sys.exit(c.exitstatus)
+
+        else:
+            logger.error(
+                "No venv instance found for %s. Use 'riot list' to get a list of valid numbers.",
+                ident,
+            )
+
     @classmethod
     def run_cmd_venv(
         cls,
