@@ -17,6 +17,8 @@ venv = Venv(
         Venv(
             pkgs={
                 "black": "==20.8b1",
+                # See https://github.com/psf/black/issues/2964 for incompatibility with click==8.1.0
+                "click": "<8.1.0",
             },
             venvs=[
                 Venv(
@@ -65,9 +67,9 @@ venv = Venv(
             name="docs",
             command="sphinx-build {cmdargs} -W -b html docs docs/_build/",
             pkgs={
-                "sphinx": "==3.3",
-                "sphinx-rtd-theme": "==0.5.0",
-                "sphinx-click": "==2.5.0",
+                "sphinx": "~=4.5.0",
+                "sphinx-rtd-theme": "~=1.0.0",
+                "sphinx-click": "~=3.1.0",
                 "reno": latest,
             },
         ),
