@@ -1,5 +1,6 @@
 import contextlib
 import os
+import re
 import shutil
 import typing
 
@@ -61,6 +62,7 @@ def test_list_with_venv_pattern(cli: click.testing.CliRunner) -> None:
         result = cli.invoke(
             riot.cli.main,
             [
+                "-P",
                 "list",
                 "test",
                 "--venv-pattern",
@@ -70,7 +72,10 @@ def test_list_with_venv_pattern(cli: click.testing.CliRunner) -> None:
         if result.exception:
             raise result.exception
         assert result.exit_code == 0, result.stdout
-        assert result.stdout == "test  Python Interpreter(_hint='3') 'pytest==5.4.3'\n"
+        assert (
+            result.stdout
+            == "[#0]  4375064  test          Interpreter(_hint='3') Packages('pytest==5.4.3')\n"
+        )
 
 
 def test_list_with_python(cli: click.testing.CliRunner) -> None:
@@ -186,8 +191,16 @@ def test_run_no_venv_pattern(cli: click.testing.CliRunner) -> None:
             ],
         )
         assert result.exit_code == 0
-        assert "✓ test:  pythonInterpreter(_hint='3') 'pytest==5.4.3'" in result.stdout
-        assert "✓ test:  pythonInterpreter(_hint='3') 'pytest'" in result.stdout
+        assert re.search(
+            r"✓ test: \[[0-9a-f]{7}\]  pythonInterpreter\(_hint='3'\) 'pytest==5.4.3'",
+            result.stdout,
+            re.MULTILINE,
+        )
+        assert re.search(
+            r"✓ test: \[[0-9a-f]{7}\]  pythonInterpreter\(_hint='3'\) 'pytest'",
+            result.stdout,
+            re.MULTILINE,
+        )
         assert "2 passed with 0 warnings, 0 failed" in result.stdout
 
 
@@ -373,14 +386,14 @@ def test_failure():
             riot.cli.main, ["run", "-s", "success"], catch_exceptions=False
         )
         assert result.exit_code == 0
-        assert "✓ success" in result.stdout
+        assert re.search(r"✓ success: \[[0-9a-f]{7}\]", result.stdout)
         assert "1 passed with 0 warnings, 0 failed" in result.stdout
 
         result = cli.invoke(
             riot.cli.main, ["run", "-s", "failure"], catch_exceptions=False
         )
         assert result.exit_code == 1
-        assert "x failure" in result.stdout
+        assert re.search(r"x failure: \[[0-9a-f]{7}\]", result.stdout)
         assert "0 passed with 0 warnings, 1 failed" in result.stdout
 
 
@@ -412,13 +425,13 @@ venv = Venv(
             riot.cli.main, ["run", "-s", "success"], catch_exceptions=False
         )
         assert result.exit_code == 0
-        assert "✓ success" in result.stdout
+        assert re.search(r"✓ success: \[[0-9a-f]{7}\]", result.stdout)
 
         result = cli.invoke(
             riot.cli.main, ["run", "-s", "success2"], catch_exceptions=False
         )
         assert result.exit_code == 0
-        assert "✓ success2" in result.stdout
+        assert re.search(r"✓ success2: \[[0-9a-f]{7}\]", result.stdout)
 
 
 def test_env(cli: click.testing.CliRunner) -> None:
@@ -454,6 +467,53 @@ def test_success():
             """
             )
 
+        result = cli.invoke(
+            riot.cli.main, ["run", "-s", "envtest"], catch_exceptions=False
+        )
+        assert result.exit_code == 0
+        assert re.search(r"✓ envtest: \[[0-9a-f]{7}\]", result.stdout)
+
+
+def test_default_env(cli: click.testing.CliRunner) -> None:
+    with cli.isolated_filesystem():
+        with open("riotfile.py", "w") as f:
+            f.write(
+                """
+from riot import Venv, latest
+
+venv = Venv(
+    pkgs={
+        "pytest": latest,
+    },
+    venvs=[
+        Venv(
+            pys=[3],
+            pkgs={"packaging": ">=21.3"},
+            name="envtest",
+            command="pytest",
+        ),
+    ],
+)
+            """
+            )
+
+        with open("test_success.py", "w") as f:
+            f.write(
+                """
+import os
+
+def test_success():
+    assert os.environ["RIOT"] == "1"
+    assert os.environ["RIOT_PYTHON_HINT"] == "Interpreter(_hint='3')"
+    assert os.environ["RIOT_PYTHON_VERSION"].startswith("3.")
+    assert os.environ["RIOT_VENV_HASH"] == "f8691e0"
+    assert os.environ["RIOT_VENV_IDENT"] == "packaging213"
+    assert os.environ["RIOT_VENV_NAME"] == "envtest"
+    assert os.environ["RIOT_VENV_PKGS"] == "'packaging>=21.3'"
+    assert os.environ["RIOT_VENV_FULL_PKGS"] == "'pytest' 'packaging>=21.3'"
+            """
+            )
+
         result = cli.invoke(
             riot.cli.main, ["run", "-s", "envtest"], catch_exceptions=False
         )
@@ -500,7 +560,7 @@ def test_success():
             riot.cli.main, ["run", "-s", "envtest"], catch_exceptions=False
         )
         assert result.exit_code == 0
-        assert "✓ envtest" in result.stdout
+        assert re.search(r"✓ envtest: \[[0-9a-f]{7}\]", result.stdout)
 
 
 def test_bad_riotfile_name(cli: click.testing.CliRunner) -> None:
@@ -545,3 +605,74 @@ this is invalid syntax
         assert result.exit_code == 1
         assert "Failed to parse" in result.stdout
         assert "SyntaxError: invalid syntax" in result.stdout
+
+
+def test_run_pass_env(
+    cli: click.testing.CliRunner, monkeypatch: _pytest.monkeypatch.MonkeyPatch
+) -> None:
+    with cli.isolated_filesystem():
+        with open("riotfile.py", "w") as f:
+            f.write(
+                """
+from riot import Venv
+
+
+venv = Venv(
+    pys=[3],
+    pkgs={
+        "pytest": [""],
+    },
+    venvs=[
+        Venv(
+            name="no_env",
+            command="pytest test_no_env.py",
+        ),
+        Venv(
+            name="env",
+            command="pytest test_env.py",
+        ),
+    ],
+)
+                """
+            )
+
+        with open("test_no_env.py", "w") as f:
+            f.write(
+                """
+import os
+
+
+def test_no_env():
+    assert os.environ.get("TEST_ENV_VAR") is None
+            """
+            )
+
+        with open("test_env.py", "w") as f:
+            f.write(
+                """
+import os
+
+
+def test_env():
+    assert os.environ.get("TEST_ENV_VAR") == "1"
+            """
+            )
+
+        # set environment variables to check in test execution
+        monkeypatch.setenv("TEST_ENV_VAR", "1")
+
+        result = cli.invoke(
+            riot.cli.main, ["run", "-s", "no_env"], catch_exceptions=False
+        )
+        assert result.exit_code == 0
+        assert "1 passed with 0 warnings, 0 failed" in result.stdout
+
+        result = cli.invoke(riot.cli.main, ["run", "-s", "env"], catch_exceptions=False)
+        assert result.exit_code == 1
+        assert "0 passed with 0 warnings, 1 failed" in result.stdout
+
+        result = cli.invoke(
+            riot.cli.main, ["run", "-s", "--pass-env", "env"], catch_exceptions=False
+        )
+        assert result.exit_code == 0
+        assert "1 passed with 0 warnings, 0 failed" in result.stdout
