@@ -13,10 +13,14 @@ _T_Path = Union[str, "os.PathLike[Any]"]
 
 
 def run(
-    args: Union[str, Sequence[str]], cwd: _T_Path, env: Optional[Dict[str, str]] = None
+    args: Union[str, Sequence[str]],
+    cwd: _T_Path,
+    env: Optional[Dict[str, str]] = None,
+    input: Optional[str] = None,  # noqa
 ) -> _T_CompletedProcess:
     return subprocess.run(
         args,
+        input=input,
         env=env,
         encoding=sys.getdefaultencoding(),
         stdout=subprocess.PIPE,
@@ -32,6 +36,7 @@ class _T_TmpRun(Protocol):
         args: Union[str, Sequence[str]],
         cwd: Optional[_T_Path] = None,
         env: Optional[Dict[str, str]] = None,
+        input: Optional[str] = None,  # noqa
     ) -> _T_CompletedProcess:
         ...
 
@@ -44,10 +49,11 @@ def tmp_run(tmp_path: pathlib.Path) -> Generator[_T_TmpRun, None, None]:
         args: Union[str, Sequence[str]],
         cwd: Optional[_T_Path] = None,
         env: Optional[Dict[str, str]] = None,
+        input: Optional[str] = None,  # noqa
     ) -> _T_CompletedProcess:
         if cwd is None:
             cwd = tmp_path
-        return run(args, cwd, env)
+        return run(args, cwd, env, input)
 
     yield _run
 
@@ -63,6 +69,7 @@ Options:
   -f, --file PATH  [default: riotfile.py]
   -v, --verbose
   -d, --debug
+  -P, --pipe       Pipe mode. Makes riot emit plain output.
   --version        Show the version and exit.
   --help           Show this message and exit.
 
@@ -70,12 +77,13 @@ Commands:
   generate  Generate base virtual environments.
   list      List all virtual env instances matching a pattern.
   run       Run virtualenv instances with names matching a pattern.
+  shell     Launch a shell inside a venv.
 """.lstrip()
     )
     assert result.stderr == ""
     assert result.returncode == 0
 
-    result = tmp_run("riot list")
+    result = tmp_run("riot -P list")
     assert (
         result.stderr
         == """
@@ -154,6 +162,7 @@ Options:
   -f, --file PATH  [default: riotfile.py]
   -v, --verbose
   -d, --debug
+  -P, --pipe       Pipe mode. Makes riot emit plain output.
   --version        Show the version and exit.
   --help           Show this message and exit.
 
@@ -161,6 +170,7 @@ Commands:
   generate  Generate base virtual environments.
   list      List all virtual env instances matching a pattern.
   run       Run virtualenv instances with names matching a pattern.
+  shell     Launch a shell inside a venv.
 """.lstrip()
     )
     assert result.stderr == ""
@@ -175,7 +185,7 @@ def test_version(tmp_run: _T_TmpRun) -> None:
 
 
 def test_list_no_file_empty_file(tmp_path: pathlib.Path, tmp_run: _T_TmpRun) -> None:
-    result = tmp_run("riot list")
+    result = tmp_run("riot -P list")
     assert (
         result.stderr
         == """
@@ -193,7 +203,7 @@ Error: Invalid value for '-f' / '--file': Path 'riotfile.py' does not exist.
 from riot import Venv
 """,
     )
-    result = tmp_run("riot list")
+    result = tmp_run("riot -P list")
     assert result.stderr == ""
     assert result.stdout == ""
     assert result.returncode == 0
@@ -211,9 +221,12 @@ venv = Venv(
 )
 """,
     )
-    result = tmp_run("riot list")
+    result = tmp_run("riot -P list")
     assert result.stderr == ""
-    assert result.stdout == "test  Python Interpreter(_hint='3') \n"
+    assert (
+        result.stdout
+        == "[#0]  1c31170  test          Interpreter(_hint='3') Packages()\n"
+    )
     assert result.returncode == 0
 
     rf_path.write_text(
@@ -229,12 +242,12 @@ venv = Venv(
 )
 """,
     )
-    result = tmp_run("riot list")
+    result = tmp_run("riot -P list")
     assert result.stderr == ""
     assert re.search(
         r"""
-test  .* 'pkg1==1.0'
-test  .* 'pkg1==2.0'
+\[\#0\]  [0-9a-f]{7}  test  .* Packages\('pkg1==1.0'\)
+\[\#1\]  [0-9a-f]{7}  test  .* Packages\('pkg1==2.0'\)
 """.lstrip(),
         result.stdout,
     )
@@ -254,14 +267,14 @@ venv = Venv(
 )
 """,
     )
-    result = tmp_run("riot list")
+    result = tmp_run("riot -P list")
     assert result.stderr == ""
     assert re.search(
         r"""
-test  .* 'pkg1==1.0' 'pkg2==2.0'
-test  .* 'pkg1==1.0' 'pkg2==3.0'
-test  .* 'pkg1==2.0' 'pkg2==2.0'
-test  .* 'pkg1==2.0' 'pkg2==3.0'
+\[\#0\]  [0-9a-f]{7}  test  .* Packages\('pkg1==1.0' 'pkg2==2.0'\)
+\[\#1\]  [0-9a-f]{7}  test  .* Packages\('pkg1==1.0' 'pkg2==3.0'\)
+\[\#2\]  [0-9a-f]{7}  test  .* Packages\('pkg1==2.0' 'pkg2==2.0'\)
+\[\#3\]  [0-9a-f]{7}  test  .* Packages\('pkg1==2.0' 'pkg2==3.0'\)
 """.lstrip(),
         result.stdout,
     )
@@ -293,18 +306,18 @@ venv = Venv(
 )
 """,
     )
-    result = tmp_run("riot list")
+    result = tmp_run("riot -P list")
     assert result.stderr == ""
     assert re.search(
         r"""
-test1  .* 'pkg1==1.0' 'pkg2==3.0'
-test1  .* 'pkg1==1.0' 'pkg2==4.0'
-test1  .* 'pkg1==2.0' 'pkg2==3.0'
-test1  .* 'pkg1==2.0' 'pkg2==4.0'
-test2  .* 'pkg1==1.0' 'pkg2==3.0'
-test2  .* 'pkg1==1.0' 'pkg2==4.0'
-test2  .* 'pkg1==2.0' 'pkg2==3.0'
-test2  .* 'pkg1==2.0' 'pkg2==4.0'
+\[\#0\]  [0-9a-f]{7}  test1  .* Packages\('pkg1==1.0' 'pkg2==3.0'\)
+\[\#1\]  [0-9a-f]{7}  test1  .* Packages\('pkg1==1.0' 'pkg2==4.0'\)
+\[\#2\]  [0-9a-f]{7}  test1  .* Packages\('pkg1==2.0' 'pkg2==3.0'\)
+\[\#3\]  [0-9a-f]{7}  test1  .* Packages\('pkg1==2.0' 'pkg2==4.0'\)
+\[\#4\]  [0-9a-f]{7}  test2  .* Packages\('pkg1==1.0' 'pkg2==3.0'\)
+\[\#5\]  [0-9a-f]{7}  test2  .* Packages\('pkg1==1.0' 'pkg2==4.0'\)
+\[\#6\]  [0-9a-f]{7}  test2  .* Packages\('pkg1==2.0' 'pkg2==3.0'\)
+\[\#7\]  [0-9a-f]{7}  test2  .* Packages\('pkg1==2.0' 'pkg2==4.0'\)
 """.lstrip(),
         result.stdout,
     )
@@ -323,7 +336,7 @@ venv = Venv(
 )
 """,
     )
-    result = tmp_run("riot list test")
+    result = tmp_run("riot -P list test")
     assert result.stderr == ""
     assert re.search(r"test .*", result.stdout)
     assert result.returncode == 0
@@ -342,14 +355,14 @@ venv = Venv(
 )
 """,
     )
-    result = tmp_run("riot list test")
+    result = tmp_run("riot -P list test")
     assert result.stderr == ""
     assert re.search(
         r"""
-test  .* 'pkg1==1.0' 'pkg2==2.0'
-test  .* 'pkg1==1.0' 'pkg2==3.0'
-test  .* 'pkg1==2.0' 'pkg2==2.0'
-test  .* 'pkg1==2.0' 'pkg2==3.0'
+\[\#0\]  [0-9a-f]{7}  test  .* Packages\('pkg1==1.0' 'pkg2==2.0'\)
+\[\#1\]  [0-9a-f]{7}  test  .* Packages\('pkg1==1.0' 'pkg2==3.0'\)
+\[\#2\]  [0-9a-f]{7}  test  .* Packages\('pkg1==2.0' 'pkg2==2.0'\)
+\[\#3\]  [0-9a-f]{7}  test  .* Packages\('pkg1==2.0' 'pkg2==3.0'\)
 """.lstrip(),
         result.stdout,
     )
@@ -381,14 +394,25 @@ venv = Venv(
 )
 """,
     )
-    result = tmp_run("riot list test2")
+    result = tmp_run("riot -P list test2")
     assert result.stderr == ""
     assert re.search(
         r"""
-test2  .* 'pkg1==1.0' 'pkg2==3.0'
-test2  .* 'pkg1==1.0' 'pkg2==4.0'
-test2  .* 'pkg1==2.0' 'pkg2==3.0'
-test2  .* 'pkg1==2.0' 'pkg2==4.0'
+\[\#4\]  [0-9a-f]{7}  test2  .* Packages\('pkg1==1.0' 'pkg2==3.0'\)
+\[\#5\]  [0-9a-f]{7}  test2  .* Packages\('pkg1==1.0' 'pkg2==4.0'\)
+\[\#6\]  [0-9a-f]{7}  test2  .* Packages\('pkg1==2.0' 'pkg2==3.0'\)
+\[\#7\]  [0-9a-f]{7}  test2  .* Packages\('pkg1==2.0' 'pkg2==4.0'\)
+""".lstrip(),
+        result.stdout,
+    )
+    assert result.returncode == 0
+
+    # Listing by hash works
+    result = tmp_run("riot -P list 144c8c4")
+    assert result.stderr == ""
+    assert re.search(
+        r"""
+\[\#7\]  144c8c4  test2  .* Packages\('pkg1==2.0' 'pkg2==4.0'\)
 """.lstrip(),
         result.stdout,
     )
@@ -446,20 +470,46 @@ test_success.py .*
 ============================== 1 passed in .*s ===============================
 
 -------------------summary-------------------
-✓ pass: .*
+✓ pass: \[[0-9a-f]{7}\].*
 1 passed with 0 warnings, 0 failed\n""".lstrip(),
         result.stdout,
     ), result.stdout
     assert result.stderr == ""
 
     result = tmp_run("riot run -s fail")
-    assert (
-        "x fail:  pythonInterpreter(_hint='3') 'pytest'\n" in result.stdout
-    ), result.stdout
+    assert re.search(
+        r"x fail: \[[0-9a-f]{7}\]  pythonInterpreter\(_hint='3'\) 'pytest'",
+        result.stdout,
+        re.MULTILINE,
+    )
     assert result.stderr == ""
     assert result.returncode == 1
 
 
+def test_run_hash(tmp_path: pathlib.Path, tmp_run: _T_TmpRun) -> None:
+    rf_path = tmp_path / "riotfile.py"
+    rf_path.write_text(
+        """
+from riot import Venv
+venv = Venv(
+    pys=[3],
+    pkgs={
+        "pytest": [""],
+    },
+    venvs=[
+        Venv(
+            name="pass",
+            command="echo pass",
+        ),
+    ],
+)
+""",
+    )
+    # DEV: The hash is consistent as long as no settings change
+    result = tmp_run("riot run -s 163716e")
+    assert result.returncode == 0, result.stderr
+
+
 def test_run_cmdargs(tmp_path: pathlib.Path, tmp_run: _T_TmpRun) -> None:
     rf_path = tmp_path / "riotfile.py"
     rf_path.write_text(
@@ -506,10 +556,9 @@ venv = Venv(
 """,
     )
     result = tmp_run("riot run test")
-    assert 'ERROR: File "setup.py"' in result.stderr
-    assert "Dev install failed, aborting!" in result.stderr
-    assert result.stdout == ""
     assert result.returncode == 1
+    assert "setup.py" in result.stderr, result.stderr
+    assert "Dev install failed, aborting!" in result.stderr, result.stderr
 
 
 def test_bad_interpreter(tmp_path: pathlib.Path, tmp_run: _T_TmpRun) -> None:
@@ -531,7 +580,6 @@ FileNotFoundError: Python interpreter DNE not found
 """.strip()
         in result.stderr
     )
-    assert result.stdout == ""
     assert result.returncode == 1
 
 
@@ -586,7 +634,7 @@ venv = Venv(
 """,
     )
     result = tmp_run("riot run -s test")
-    env = dict(_.split("=") for _ in result.stdout.splitlines() if "=" in _)
+    env = dict(_.split("=", maxsplit=1) for _ in result.stdout.splitlines() if "=" in _)
     assert result.returncode == 0
 
     version = "".join((str(_) for _ in sys.version_info[:3]))
@@ -647,7 +695,7 @@ venv = Venv(
 """,
     )
     result = tmp_run("riot run -s test")
-    env = dict(_.split("=") for _ in result.stdout.splitlines() if "=" in _)
+    env = dict(_.split("=", maxsplit=1) for _ in result.stdout.splitlines() if "=" in _)
     assert result.returncode == 0, result.stderr
 
     venv_name = "venv_py{}_pytest".format(
@@ -724,12 +772,14 @@ venv = Venv(
 )
 """,
     )
-    result = tmp_run("riot -v run -s child")
-    venv_path = tmp_path / ".riot/venv_py{}_pip_pytest".format(
+    result = tmp_run("riot -Pv run -s child")
+    venv_path = tmp_path / ".riot/venv_py{}_pytest_pip".format(
         "".join((str(_) for _ in sys.version_info[:3]))
     )
-    assert f"Creating virtualenv '{venv_path}'" in result.stderr
-    assert f"Running command 'echo $PYTHONPATH' in venv '{venv_path}'"
+    assert f"Creating virtualenv '{venv_path}'" in result.stderr, result.stderr
+    assert (
+        f"Running command 'echo $PYTHONPATH' in venv '{venv_path}'" in result.stderr
+    ), result.stderr
     assert result.stdout.startswith(":".join(("", str(tmp_path))))
     assert result.returncode == 0
 
@@ -749,3 +799,67 @@ venv = Venv(
     )
     result = tmp_run("riot -v run -s test")
     assert result.returncode == 0
+
+
+def test_shell(tmp_path: pathlib.Path, tmp_run: _T_TmpRun) -> None:
+    rf_path = tmp_path / "riotfile.py"
+    rf_path.write_text(
+        """
+from riot import Venv, latest
+venv = Venv(
+    name="test",
+    pys=["3"],
+    pkgs={"requests": latest},
+    command="python -c 'import requests'",
+)
+""",
+    )
+
+    result = tmp_run("riot -vd --pipe shell '#0'", input="exit\n")
+    assert (
+        re.search(
+            r"Setting venv path to .+[.]riot/venv_py[0-9]+_requests", result.stderr
+        )
+        is None
+    )
+    assert re.search(r"Setting venv path to .+[.]riot/venv_py[0-9]+", result.stderr)
+
+    assert result.returncode == 0, result.stderr
+
+
+def test_shell_created(tmp_path: pathlib.Path, tmp_run: _T_TmpRun) -> None:
+    rf_path = tmp_path / "riotfile.py"
+    rf_path.write_text(
+        """
+from riot import Venv, latest
+venv = Venv(
+    name="test",
+    pys=["3"],
+    pkgs={"requests": latest},
+    command="python -c 'import requests'",
+    create=True,
+)
+""",
+    )
+    setup_path = tmp_path / "setup.py"
+    setup_path.write_text(
+        """
+from setuptools import setup, find_packages
+
+setup(
+    name='foo',
+    version='1.0.0',
+    url='https://github.com/bar/foo.git',
+    author='Me',
+    author_email='author@email.com',
+    description='Noop',
+    packages=find_packages(),
+    install_requires=[],
+)
+""",
+    )
+    result = tmp_run("riot -vd --pipe shell '#0'", input="exit\n")
+    assert re.search(
+        r"Setting venv path to .+[.]riot/venv_py[0-9]+_requests", result.stderr
+    )
+    assert result.returncode == 0, result.stderr
